@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2016 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -7,25 +7,24 @@
 # --
 
 package Kernel::System::UnitTest::Selenium;
+## nofilter(TidyAll::Plugin::OTRS::Perl::Goto)
 
 use strict;
 use warnings;
 
 use MIME::Base64();
-use File::Path();
 use File::Temp();
 
 use Kernel::Config;
 use Kernel::System::User;
+use Kernel::System::UnitTest::Helper;
 
 our @ObjectDependencies = (
     'Kernel::Config',
-    'Kernel::System::AuthSession',
     'Kernel::System::Log',
     'Kernel::System::Main',
     'Kernel::System::Time',
-    'Kernel::System::UnitTest::Driver',
-    'Kernel::System::UnitTest::Helper',
+    'Kernel::System::UnitTest',
 );
 
 =head1 NAME
@@ -60,9 +59,6 @@ Specify the connection details in Config.pm, like this:
         platform            => 'ANY',
         window_height       => 1200,    # optional, default 1000
         window_width        => 1600,    # optional, default 1200
-        extra_capabilities => {
-            marionette     => \0,   # Required to run FF 47 or older on Selenium 3+.
-        },
     };
 
 Then you can use the full API of Selenium::Remote::Driver on this object.
@@ -72,15 +68,15 @@ Then you can use the full API of Selenium::Remote::Driver on this object.
 sub new {
     my ( $Class, %Param ) = @_;
 
-    $Param{UnitTestDriverObject} ||= $Kernel::OM->Get('Kernel::System::UnitTest::Driver');
+    $Param{UnitTestObject} ||= $Kernel::OM->Get('Kernel::System::UnitTest');
 
-    $Param{UnitTestDriverObject}->True( 1, "Starting up Selenium scenario..." );
+    $Param{UnitTestObject}->True( 1, "Starting up Selenium scenario..." );
 
     my %SeleniumTestsConfig = %{ $Kernel::OM->Get('Kernel::Config')->Get('SeleniumTestsConfig') // {} };
 
     if ( !%SeleniumTestsConfig ) {
         my $Self = bless {}, $Class;
-        $Self->{UnitTestDriverObject} = $Param{UnitTestDriverObject};
+        $Self->{UnitTestObject} = $Param{UnitTestObject};
         return $Self;
     }
 
@@ -100,38 +96,19 @@ sub new {
         webelement_class => 'Kernel::System::UnitTest::Selenium::WebElement',
         %SeleniumTestsConfig
     );
-    $Self->{UnitTestDriverObject} = $Param{UnitTestDriverObject};
-    $Self->{SeleniumTestsActive}  = 1;
+    $Self->{UnitTestObject}      = $Param{UnitTestObject};
+    $Self->{SeleniumTestsActive} = 1;
 
     #$Self->debug_on();
 
     # set screen size from config or use defauls
-    my $Height = $SeleniumTestsConfig{window_height} || 1000;
-    my $Width  = $SeleniumTestsConfig{window_width}  || 1200;
+    my $Height = $SeleniumTestsConfig{window_height} || 1200;
+    my $Width  = $SeleniumTestsConfig{window_width}  || 1400;
+
     $Self->set_window_size( $Height, $Width );
 
-    # get remote host with some precautions for certain unit test systems
-    my $FQDN = $Kernel::OM->Get('Kernel::Config')->Get('FQDN');
-
-    # try to resolve fqdn host
-    if ( $FQDN ne 'yourhost.example.com' && gethostbyname($FQDN) ) {
-        $Self->{BaseURL} = $FQDN;
-    }
-
-    # try to resolve localhost instead
-    if ( !$Self->{BaseURL} && gethostbyname('localhost') ) {
-        $Self->{BaseURL} = 'localhost';
-    }
-
-    # use hardcoded localhost ip address
-    if ( !$Self->{BaseURL} ) {
-        $Self->{BaseURL} = '127.0.0.1';
-    }
-
-    $Self->{BaseURL} = $Kernel::OM->Get('Kernel::Config')->Get('HttpType') . '://' . $Self->{BaseURL};
-
-    # Remember the start system time for the selenium test run.
-    $Self->{TestStartSystemTime} = $Kernel::OM->Get('Kernel::System::Time')->SystemTime();
+    $Self->{BaseURL} = $Kernel::OM->Get('Kernel::Config')->Get('HttpType') . '://';
+    $Self->{BaseURL} .= Kernel::System::UnitTest::Helper->GetTestHTTPHostname();
 
     return $Self;
 }
@@ -149,7 +126,7 @@ sub RunTest {
     my ( $Self, $Test ) = @_;
 
     if ( !$Self->{SeleniumTestsActive} ) {
-        $Self->{UnitTestDriverObject}->True( 1, 'Selenium testing is not active, skipping tests.' );
+        $Self->{UnitTestObject}->True( 1, 'Selenium testing is not active, skipping tests.' );
         return 1;
     }
 
@@ -183,10 +160,12 @@ sub _execute_command {    ## no critic
         }
     );
 
-    $Self->{UnitTestDriverObject}->True(
-        1,
-        $TestName
-    );
+    if ( $Self->{SuppressCommandRecording} ) {
+        print $TestName;
+    }
+    else {
+        $Self->{UnitTestObject}->True( 1, $TestName );
+    }
 
     return $Result;
 }
@@ -213,28 +192,6 @@ sub get {    ## no critic
     return;
 }
 
-=item get_alert_text()
-
-Override get_alert_text() method of base class to return alert text as string.
-
-    my $AlertText = $SeleniumObject->get_alert_text();
-
-returns
-
-    my $AlertText = 'Some alert text!'
-
-=cut
-
-sub get_alert_text {    ## no critic
-    my ($Self) = @_;
-
-    my $AlertText = $Self->SUPER::get_alert_text();
-
-    die "Alert dialog is not present" if ref $AlertText eq 'HASH';    # Chrome returns HASH when there is no alert text.
-
-    return $AlertText;
-}
-
 =item VerifiedGet()
 
 perform a get() call, but wait for the page to be fully loaded (works only within OTRS).
@@ -253,7 +210,7 @@ sub VerifiedGet {
 
     $Self->WaitFor(
         JavaScript =>
-            'return typeof(Core) == "object" && typeof(Core.App) == "object" && Core.App.PageLoadComplete'
+            'return typeof(Core) == "object" && typeof(Core.Config) == "object" && Core.Config.Get("Baselink")'
     ) || die "OTRS API verification failed after page load.";
 
     return;
@@ -275,7 +232,7 @@ sub VerifiedRefresh {
 
     $Self->WaitFor(
         JavaScript =>
-            'return typeof(Core) == "object" && typeof(Core.App) == "object" && Core.App.PageLoadComplete'
+            'return typeof(Core) == "object" && typeof(Core.Config) == "object" && Core.Config.Get("Baselink")'
     ) || die "OTRS API verification failed after page load.";
 
     return;
@@ -307,7 +264,7 @@ sub Login {
         }
     }
 
-    $Self->{UnitTestDriverObject}->True( 1, 'Initiating login...' );
+    $Self->{UnitTestObject}->True( 1, 'Initiating login...' );
 
     # we will try several times to log in
     my $MaxTries = 5;
@@ -333,13 +290,13 @@ sub Login {
             # login successful?
             $Self->find_element( 'a#LogoutButton', 'css' );    # dies if not found
 
-            $Self->{UnitTestDriverObject}->True( 1, 'Login sequence ended...' );
+            $Self->{UnitTestObject}->True( 1, 'Login sequence ended...' );
         };
 
         # an error happend
         if ($@) {
 
-            $Self->{UnitTestDriverObject}->True( 1, "Login attempt $Try of $MaxTries not successful." );
+            $Self->{UnitTestObject}->True( 1, "Login attempt $Try of $MaxTries not successful." );
 
             # try again
             next TRY if $Try < $MaxTries;
@@ -364,9 +321,10 @@ wait with increasing sleep intervals until the given condition is true or the wa
 Exactly one condition (JavaScript or WindowCount) must be specified.
 
     my $Success = $SeleniumObject->WaitFor(
-        JavaScript  => 'return $(".someclass").length',   # Javascript code that checks condition
-        WindowCount => 2,                                 # Wait until this many windows are open
-        Time        => 20,                                # optional, wait time in seconds (default 20)
+        JavaScript   => 'return $(".someclass").length',   # Javascript code that checks condition
+        AlertPresent => 1,                                 # Wait until an alert, confirm or prompt dialog is present
+        WindowCount  => 2,                                 # Wait until this many windows are open
+        Time         => 20,                                # optional, wait time in seconds (default 20)
     );
 
 =cut
@@ -374,21 +332,28 @@ Exactly one condition (JavaScript or WindowCount) must be specified.
 sub WaitFor {
     my ( $Self, %Param ) = @_;
 
-    if ( !$Param{JavaScript} && !$Param{WindowCount} ) {
-        die "Need JavaScript.";
+    if ( !$Param{JavaScript} && !$Param{WindowCount} && !$Param{AlertPresent} ) {
+        die "Need JavaScript, WindowCount or AlertPresent.";
     }
 
+    local $Self->{SuppressCommandRecording} = 1;
+
     $Param{Time} //= 20;
     my $WaitedSeconds = 0;
     my $Interval      = 0.1;
 
     while ( $WaitedSeconds <= $Param{Time} ) {
         if ( $Param{JavaScript} ) {
-            return 1 if $Self->execute_script( $Param{JavaScript} );
+            return 1 if $Self->execute_script( $Param{JavaScript} )
         }
         elsif ( $Param{WindowCount} ) {
             return 1 if scalar( @{ $Self->get_window_handles() } ) == $Param{WindowCount};
         }
+        elsif ( $Param{AlertPresent} ) {
+
+            # Eval is needed because the method would throw if no alert is present (yet).
+            return 1 if eval { $Self->get_alert_text() };
+        }
         sleep $Interval;
         $WaitedSeconds += $Interval;
         $Interval += 0.1;
@@ -396,6 +361,70 @@ sub WaitFor {
     return;
 }
 
+=item DragAndDrop()
+
+Drag and drop an element.
+
+    $SeleniumObject->DragAndDrop(
+        Element         => '.Element', # (required) css selector of element which should be dragged
+        Target          => '.Target',  # (required) css selector of element on which the dragged element should be dropped
+        TargetOffset    => {           # (optional) Offset for target. If not specified, the mouse will move to the middle of the element.
+            X   => 150,
+            Y   => 100,
+        }
+    );
+
+=cut
+
+sub DragAndDrop {
+
+    my ( $Self, %Param ) = @_;
+
+    # Value is optional parameter
+    for my $Needed (qw(Element Target)) {
+        if ( !$Param{$Needed} ) {
+            die "Need $Needed";
+        }
+    }
+
+    my %TargetOffset;
+    if ( $Param{TargetOffset} ) {
+        %TargetOffset = (
+            xoffset => $Param{TargetOffset}->{X} || 0,
+            yoffset => $Param{TargetOffset}->{Y} || 0,
+        );
+    }
+
+    # Make sure Element is visible
+    $Self->WaitFor(
+        JavaScript => 'return typeof($) === "function" && $(\'' . $Param{Element} . ':visible\').length;',
+    );
+    my $Element = $Self->find_element( $Param{Element}, 'css' );
+
+    # Move mouse to from element, drag and drop
+    $Self->mouse_move_to_location( element => $Element );
+
+    # Holds the mouse button on the element
+    $Self->button_down();
+
+    # Make sure Target is visible
+    $Self->WaitFor(
+        JavaScript => 'return typeof($) === "function" && $(\'' . $Param{Target} . ':visible\').length;',
+    );
+    my $Target = $Self->find_element( $Param{Target}, 'css' );
+
+    # Move mouse to the destination
+    $Self->mouse_move_to_location(
+        element => $Target,
+        %TargetOffset,
+    );
+
+    # Release
+    $Self->button_up();
+
+    return;
+}
+
 =item HandleError()
 
 use this method to handle any Selenium exceptions.
@@ -403,111 +432,61 @@ use this method to handle any Selenium exceptions.
     $SeleniumObject->HandleError($@);
 
 It will create a failing test result and store a screenshot of the page
-for analysis (in folder /var/otrs-unittest if it exists, in $Home/var/httpd/htdocs otherwise).
+for analysis (in folder /var/otrs-unittest if it exists, in /tmp otherwise).
 
 =cut
 
 sub HandleError {
     my ( $Self, $Error ) = @_;
 
-    $Self->{UnitTestDriverObject}->False( 1, "Exception in Selenium': $Error" );
+    $Self->{UnitTestObject}->False( 1, "Exception in Selenium': $Error" );
 
     #eval {
     my $Data = $Self->screenshot();
     return if !$Data;
     $Data = MIME::Base64::decode_base64($Data);
 
-    #
-    # Store screenshots in a local folder from where they can be opened directly in the browser.
-    #
-    my $LocalScreenshotDir = $Kernel::OM->Get('Kernel::Config')->Get('Home') . '/var/httpd/htdocs/SeleniumScreenshots';
-    mkdir $LocalScreenshotDir || return $Self->False( 1, "Could not create $LocalScreenshotDir." );
+    my $TmpDir = -d '/var/otrs-unittest/' ? '/var/otrs-unittest/' : '/tmp/';
+    $TmpDir .= 'SeleniumScreenshots/';
+    mkdir $TmpDir || return $Self->False( 1, "Could not create $TmpDir." );
+
+    my $Product = $Self->{UnitTestObject}->{Product};
+    $Product =~ s{[^a-z0-9_.\-]+}{_}smxig;
+    $TmpDir .= $Product;
+    mkdir $TmpDir || return $Self->False( 1, "Could not create $TmpDir." );
 
     my $Filename = $Kernel::OM->Get('Kernel::System::Time')->CurrentTimestamp();
     $Filename .= '-' . ( int rand 100_000_000 ) . '.png';
     $Filename =~ s{[ :]}{-}smxg;
 
-    my $HttpType = $Kernel::OM->Get('Kernel::Config')->Get('HttpType');
-    my $Hostname = $Kernel::OM->Get('Kernel::System::UnitTest::Helper')->GetTestHTTPHostname();
-    my $URL      = "$HttpType://$Hostname/"
-        . $Kernel::OM->Get('Kernel::Config')->Get('Frontend::WebPath')
-        . "SeleniumScreenshots/$Filename";
-
     $Kernel::OM->Get('Kernel::System::Main')->FileWrite(
-        Directory => $LocalScreenshotDir,
+        Directory => $TmpDir,
         Filename  => $Filename,
         Content   => \$Data,
-    ) || return $Self->False( 1, "Could not write file $LocalScreenshotDir/$Filename" );
+    ) || return $Self->False( 1, "Could not write file $TmpDir/$Filename" );
 
-    #
-    # If a shared screenshot folder is present, then we also store the screenshot there for external use.
-    #
-    if ( -d '/var/otrs-unittest/' ) {
-
-        my $SharedScreenshotDir = '/var/otrs-unittest/SeleniumScreenshots';
-        mkdir $SharedScreenshotDir || return $Self->False( 1, "Could not create $SharedScreenshotDir." );
-
-        $Kernel::OM->Get('Kernel::System::Main')->FileWrite(
-            Directory => $SharedScreenshotDir,
-            Filename  => $Filename,
-            Content   => \$Data,
-            )
-            || return $Self->{UnitTestDriverObject}->False( 1, "Could not write file $SharedScreenshotDir/$Filename" );
-    }
-
-    $Self->{UnitTestDriverObject}->False( 1, "Saved screenshot in $URL" );
-    $Self->{UnitTestDriverObject}->AttachSeleniumScreenshot(
-        Filename => $Filename,
-        Content  => $Data
+    $Self->{UnitTestObject}->False(
+        1,
+        "Saved screenshot in file://$TmpDir/$Filename",
     );
 }
 
-=head2 DEMOLISH()
+=item DESTROY()
 
-override DEMOLISH from L<Selenium::Remote::Driver> (required because this class is managed by L<Moo>).
-Adds a unit test result to indicate the shutdown, and performs some cleanups.
+cleanup. Adds a unit test result to indicate the shutdown.
 
 =cut
 
-sub DEMOLISH {
+sub DESTROY {
     my $Self = shift;
 
     # Could be missing on early die.
-    if ( $Self->{UnitTestDriverObject} ) {
-        $Self->{UnitTestDriverObject}->True( 1, "Shutting down Selenium scenario." );
+    if ( $Self->{UnitTestObject} ) {
+        $Self->{UnitTestObject}->True( 1, "Shutting down Selenium scenario." );
     }
 
     if ( $Self->{SeleniumTestsActive} ) {
-        $Self->SUPER::DEMOLISH(@_);
-
-        # Cleanup possibly leftover zombie firefox profiles.
-        my @LeftoverFirefoxProfiles = $Kernel::OM->Get('Kernel::System::Main')->DirectoryRead(
-            Directory => '/tmp/',
-            Filter    => 'anonymous*webdriver-profile',
-        );
-
-        for my $LeftoverFirefoxProfile (@LeftoverFirefoxProfiles) {
-            if ( -d $LeftoverFirefoxProfile ) {
-                File::Path::remove_tree($LeftoverFirefoxProfile);
-            }
-        }
-
-        # Cleanup all sessions, which was created after the selenium test start time.
-        my $AuthSessionObject = $Kernel::OM->Get('Kernel::System::AuthSession');
-
-        my @Sessions = $AuthSessionObject->GetAllSessionIDs();
-
-        SESSION:
-        for my $SessionID (@Sessions) {
-
-            my %SessionData = $AuthSessionObject->GetSessionIDData( SessionID => $SessionID );
-
-            next SESSION if !%SessionData;
-            next SESSION
-                if $SessionData{UserSessionStart} && $SessionData{UserSessionStart} < $Self->{TestStartSystemTime};
-
-            $AuthSessionObject->RemoveSessionID( SessionID => $SessionID );
-        }
+        $Self->SUPER::DESTROY();
     }
 }
 
